@@ -1,3 +1,3 @@
 class TestMemory < ActiveRecord::Base
-  set_table_name 'test_memory'
+  self.table_name = 'test_memory'
 end
