@@ -19,7 +19,7 @@ module ActiveRecord::Import::PostgreSQLAdapter
     sql2insert = base_sql + values.join( ',' ) + post_sql
 
     columns = returning_columns(options)
-    if columns.blank? || options[:no_returning]
+    if columns.blank? || (options[:no_returning] && !options[:recursive])
       insert( sql2insert, *args )
     else
       returned_values = if columns.size > 1
@@ -73,14 +73,14 @@ module ActiveRecord::Import::PostgreSQLAdapter
       if (options[:ignore] || options[:on_duplicate_key_ignore]) && !options[:on_duplicate_key_update] && !options[:recursive]
         sql << sql_for_on_duplicate_key_ignore( table_name, options[:on_duplicate_key_ignore] )
       end
-    elsif options[:on_duplicate_key_ignore] && !options[:on_duplicate_key_update]
+    elsif logger && options[:on_duplicate_key_ignore] && !options[:on_duplicate_key_update]
       logger.warn "Ignoring on_duplicate_key_ignore because it is not supported by the database."
     end
 
     sql += super(table_name, options)
 
     columns = returning_columns(options)
-    unless columns.blank? || options[:no_returning]
+    unless columns.blank? || (options[:no_returning] && !options[:recursive])
       sql << " RETURNING \"#{columns.join('", "')}\""
     end
 
@@ -158,7 +158,7 @@ module ActiveRecord::Import::PostgreSQLAdapter
       qc = quote_column_name( column )
       "#{qc}=EXCLUDED.#{qc}"
     end
-    increment_locking_column!(results, locking_column)
+    increment_locking_column!(table_name, results, locking_column)
     results.join( ',' )
   end
 
@@ -168,7 +168,7 @@ module ActiveRecord::Import::PostgreSQLAdapter
       qc2 = quote_column_name( column2 )
       "#{qc1}=EXCLUDED.#{qc2}"
     end
-    increment_locking_column!(results, locking_column)
+    increment_locking_column!(table_name, results, locking_column)
     results.join( ',' )
   end
 
@@ -195,17 +195,17 @@ module ActiveRecord::Import::PostgreSQLAdapter
     exception.is_a?(ActiveRecord::StatementInvalid) && exception.to_s.include?('duplicate key')
   end
 
-  def supports_on_duplicate_key_update?(current_version = postgresql_version)
-    current_version >= MIN_VERSION_FOR_UPSERT
+  def supports_on_duplicate_key_update?
+    database_version >= MIN_VERSION_FOR_UPSERT
   end
 
   def supports_setting_primary_key_of_imported_objects?
     true
   end
 
-  def increment_locking_column!(results, locking_column)
-    if locking_column.present?
-      results << "\"#{locking_column}\"=EXCLUDED.\"#{locking_column}\"+1"
-    end
+  private
+
+  def database_version
+    defined?(postgresql_version) ? postgresql_version : super
   end
 end
