@@ -26,6 +26,7 @@ module ActiveRecord::Import #:nodoc:
   class Validator
     def initialize(klass, options = {})
       @options = options
+      @validator_class = klass
       init_validations(klass)
     end
 
@@ -70,6 +71,8 @@ module ActiveRecord::Import #:nodoc:
     end
 
     def valid_model?(model)
+      init_validations(model.class) unless model.class == @validator_class
+
       validation_context = @options[:validate_with_context]
       validation_context ||= (model.new_record? ? :create : :update)
       current_context = model.send(:validation_context)
@@ -242,16 +245,17 @@ class ActiveRecord::Associations::CollectionAssociation
   alias import bulk_import unless respond_to? :import
 end
 
+module ActiveRecord::Import::Connection
+  def establish_connection(args = nil)
+    conn = super(args)
+    ActiveRecord::Import.load_from_connection_pool connection_pool
+    conn
+  end
+end
+
 class ActiveRecord::Base
   class << self
-    def establish_connection_with_activerecord_import(*args)
-      conn = establish_connection_without_activerecord_import(*args)
-      ActiveRecord::Import.load_from_connection_pool connection_pool
-      conn
-    end
-
-    alias establish_connection_without_activerecord_import establish_connection
-    alias establish_connection establish_connection_with_activerecord_import
+    prepend ActiveRecord::Import::Connection
 
     # Returns true if the current database connection adapter
     # supports import functionality, otherwise returns false.
@@ -528,7 +532,7 @@ class ActiveRecord::Base
         import_helper(*args)
       end
     end
-    alias import bulk_import unless respond_to? :import
+    alias import bulk_import unless ActiveRecord::Base.respond_to? :import
 
     # Imports a collection of values if all values are valid. Import fails at the
     # first encountered validation error and raises ActiveRecord::RecordInvalid
@@ -540,7 +544,7 @@ class ActiveRecord::Base
 
       bulk_import(*args, options)
     end
-    alias import! bulk_import! unless respond_to? :import!
+    alias import! bulk_import! unless ActiveRecord::Base.respond_to? :import!
 
     def import_helper( *args )
       options = { validate: true, timestamps: true }
@@ -574,20 +578,11 @@ class ActiveRecord::Base
           end
         end
 
-        default_values = column_defaults
-        stored_attrs = respond_to?(:stored_attributes) ? stored_attributes : {}
-        serialized_attrs = if defined?(ActiveRecord::Type::Serialized)
-          attrs = column_names.select { |c| type_for_attribute(c.to_s).class == ActiveRecord::Type::Serialized }
-          Hash[attrs.map { |a| [a, nil] }]
-        else
-          serialized_attributes
-        end
-
         update_attrs = if record_timestamps && options[:timestamps]
           if respond_to?(:timestamp_attributes_for_update, true)
             send(:timestamp_attributes_for_update).map(&:to_sym)
           else
-            new.send(:timestamp_attributes_for_update_in_model)
+            allocate.send(:timestamp_attributes_for_update_in_model)
           end
         end
 
@@ -608,12 +603,8 @@ class ActiveRecord::Base
                update_attrs && update_attrs.include?(name.to_sym) &&
                !model.send("#{name}_changed?")
               nil
-            elsif stored_attrs.key?(name.to_sym) ||
-                  serialized_attrs.key?(name.to_s) ||
-                  default_values[name.to_s]
-              model.read_attribute(name.to_s)
             else
-              model.read_attribute_before_type_cast(name.to_s)
+              model.read_attribute(name.to_s)
             end
           end
         end
@@ -640,7 +631,7 @@ class ActiveRecord::Base
         end
         # supports empty array
       elsif args.last.is_a?( Array ) && args.last.empty?
-        return ActiveRecord::Import::Result.new([], 0, [])
+        return ActiveRecord::Import::Result.new([], 0, [], [])
         # supports 2-element array and array
       elsif args.size == 2 && args.first.is_a?( Array ) && args.last.is_a?( Array )
 
@@ -712,8 +703,7 @@ class ActiveRecord::Base
             # keep track of the instance and the position it is currently at. if this fails
             # validation we'll use the index to remove it from the array_of_attributes
             arr.each_with_index do |hsh, i|
-              model = new
-              hsh.each_pair { |k, v| model[k] = v }
+              model = new(hsh)
               next if validator.valid_model?(model)
               raise(ActiveRecord::RecordInvalid, model) if options[:raise_error]
               array_of_attributes[i] = nil
@@ -851,6 +841,19 @@ class ActiveRecord::Base
         end
       end
 
+      deserialize_value = lambda do |column, value|
+        column = columns_hash[column]
+        return value unless column
+        if respond_to?(:type_caster)
+          type = type_for_attribute(column.name)
+          type.deserialize(value)
+        elsif column.respond_to?(:type_cast_from_database)
+          column.type_cast_from_database(value)
+        else
+          value
+        end
+      end
+
       if models.size == import_result.results.size
         columns = Array(options[:returning])
         single_column = "#{columns.first}=" if columns.size == 1
@@ -858,17 +861,22 @@ class ActiveRecord::Base
           model = models[index]
 
           if single_column
-            model.send(single_column, result)
+            val = deserialize_value.call(columns.first, result)
+            model.send(single_column, val)
           else
             columns.each_with_index do |column, col_index|
-              model.send("#{column}=", result[col_index])
+              val = deserialize_value.call(column, result[col_index])
+              model.send("#{column}=", val)
             end
           end
         end
       end
 
       models.each do |model|
-        if model.respond_to?(:clear_changes_information) # Rails 4.0 and higher
+        if model.respond_to?(:changes_applied) # Rails 4.1.8 and higher
+          model.changes_internally_applied if model.respond_to?(:changes_internally_applied) # legacy behavior for Rails 5.1
+          model.changes_applied
+        elsif model.respond_to?(:clear_changes_information) # Rails 4.0 and higher
           model.clear_changes_information
         else # Rails 3.2
           model.instance_variable_get(:@changed_attributes).clear
@@ -879,10 +887,12 @@ class ActiveRecord::Base
 
     # Sync belongs_to association ids with foreign key field
     def load_association_ids(model)
+      changed_columns = model.changed
       association_reflections = model.class.reflect_on_all_associations(:belongs_to)
       association_reflections.each do |association_reflection|
         column_name = association_reflection.foreign_key
         next if association_reflection.options[:polymorphic]
+        next if changed_columns.include?(column_name)
         association = model.association(association_reflection.name)
         association = association.target
         next if association.blank? || model.public_send(column_name).present?
@@ -901,8 +911,9 @@ class ActiveRecord::Base
       associated_objects_by_class = {}
       models.each { |model| find_associated_objects_for_import(associated_objects_by_class, model) }
 
-      # :on_duplicate_key_update not supported for associations
+      # :on_duplicate_key_update and :returning not supported for associations
       options.delete(:on_duplicate_key_update)
+      options.delete(:returning)
 
       associated_objects_by_class.each_value do |associations|
         associations.each_value do |associated_records|
@@ -961,7 +972,7 @@ class ActiveRecord::Base
           elsif column
             if respond_to?(:type_caster)                                         # Rails 5.0 and higher
               type = type_for_attribute(column.name)
-              val = type.type == :boolean ? type.cast(val) : type.serialize(val)
+              val = !type.respond_to?(:subtype) && type.type == :boolean ? type.cast(val) : type.serialize(val)
               connection_memo.quote(val)
             elsif column.respond_to?(:type_cast_from_user)                       # Rails 4.2
               connection_memo.quote(column.type_cast_from_user(val), column)
@@ -970,9 +981,11 @@ class ActiveRecord::Base
                 val = serialized_attributes[column.name].dump(val)
               end
               # Fixes #443 to support binary (i.e. bytea) columns on PG
-              val = column.type_cast(val) unless column.type.to_sym == :binary
+              val = column.type_cast(val) unless column.type && column.type.to_sym == :binary
               connection_memo.quote(val, column)
             end
+          else
+            raise ArgumentError, "Number of values (#{arr.length}) exceeds number of columns (#{columns.length})"
           end
         end
         "(#{my_values.join(',')})"
@@ -987,7 +1000,7 @@ class ActiveRecord::Base
         timestamp_columns[:create] = timestamp_attributes_for_create_in_model
         timestamp_columns[:update] = timestamp_attributes_for_update_in_model
       else
-        instance = new
+        instance = allocate
         timestamp_columns[:create] = instance.send(:timestamp_attributes_for_create_in_model)
         timestamp_columns[:update] = instance.send(:timestamp_attributes_for_update_in_model)
       end
