@@ -1,6 +1,6 @@
 require File.expand_path(File.dirname(__FILE__) + '/../test_helper')
 
-require File.expand_path(File.dirname(__FILE__) + '/../support/mysql/assertions')
+require File.expand_path(File.dirname(__FILE__) + '/../support/assertions')
 require File.expand_path(File.dirname(__FILE__) + '/../support/mysql/import_examples')
 
-should_support_mysql_import_functionality
\ No newline at end of file
+should_support_mysql_import_functionality
